#include "arm_task.h"
#include "rm_hal_lib.h"
#include "math.h"
#include "cmsis_os.h"
#include "uart_device.h"
#include "sys.h"

float arm_procotol[2] = {100, 0};
float arm_pos[2] = {100, -15};
servo_ctrl_t servo = {0};
int last_rc_sw = -1;

int type=0;

void arm_init(void)
{
	set_pwm_group_param(PWM_GROUP1, 20000);
	set_pwm_group_param(PWM_GROUP2, 20000);

	start_pwm_output(ARM_S1_IO);
	start_pwm_output(ARM_S2_IO);
	start_pwm_output(ARM_S3_IO);
	start_pwm_output(ARM_S4_IO);
	
	start_pwm_output(PWM_IO5);

	servo.s_s1 = ARM_S1_INIT;
	servo.s_s2 = ARM_S2_INIT;
	servo.s_s3 = ARM_S3_INIT;
	servo.s_s4 = ARM_S4_INIT;
	
	servo.s_camera = 0;

	servo_ctrl(&servo);
}

void arm_angle_calc(float *pos, int *angle) // pos (mm)   angle (degree)
{
	double angle1, angle2, a1, a2, b1, b2;
	int16_t r1 = ARM_L1, r2 = ARM_L1, x = pos[0], y = pos[1];
	a1 = r1 * r1 + x * x + y * y - r2 * r2;
	b1 = 2 * r1 * sqrt(x * x + y * y);
	a2 = r2 * r2 + x * x + y * y - r1 * r1;
	b2 = 2 * r2 * sqrt(x * x + y * y);

	angle1 = (acos(a1 / b1) + atan(1.0 * y / x)) * DGR_RAD;
	angle2 = (PI - acos(a2 / b2) + atan(1.0 * y / x)) * DGR_RAD;

	angle[0] = 2000 * (angle1 - 22) / SERVO_MAX_ANGLE + 500;
	angle[1] = 2000 * (180.0 - angle2) / SERVO_MAX_ANGLE + 500;
}

void arm_pos_ctrl(float *pos)
{
	VAL_LIMIT(pos[0], ARM_MIN_Y, ARM_MAX_Y);
	VAL_LIMIT(pos[1], ARM_MIN_Z, ARM_MAX_Z);

	int angle[2] = {0};
	arm_angle_calc(pos, angle);

	servo.s_s1 = angle[0];
	servo.s_s2 = angle[1];

	servo_ctrl(&servo);
	osDelay(50);
}
/*

void arm_claw_ctrl(arm_claw_e state)
{
	uint16_t delay = 50;
	uint16_t pos = 0;

	if (state == ARM_CLAW_CLOSE)
	{
		for (pos = ARM_CLAW_OPEN_ANGLE; pos > ARM_CLAW_CLOSE_ANGLE; pos -= 30)
		{
			servo.s_s3 = pos;
			servo_ctrl(&servo);
			osDelay(delay);
		}
	}

	if (state == ARM_CLAW_OPEN)
	{
		for (pos = ARM_CLAW_CLOSE_ANGLE; pos < ARM_CLAW_OPEN_ANGLE; pos += 30)
		{
			servo.s_s3 = pos;
			servo_ctrl(&servo);
			osDelay(delay);
		}
	}
}
*/
void servo_ctrl(servo_ctrl_t *servo_ctrl)
{
	set_pwm_param(ARM_S1_IO, servo_ctrl->s_s1);
	set_pwm_param(ARM_S2_IO, servo_ctrl->s_s2);
	set_pwm_param(ARM_S3_IO, servo_ctrl->s_s3);
	set_pwm_param(ARM_S4_IO, servo_ctrl->s_s4);
	set_pwm_param(PWM_IO5, servo_ctrl->s_camera);
}

void arm_remote_information_get(void) {
	
	VAL_LIMIT(arm_pos[0], ARM_MIN_Y, ARM_MAX_Y);
	VAL_LIMIT(arm_pos[1], ARM_MIN_Z, ARM_MAX_Z);
	
	if(rc.sw1==RC_MI)
	{
	arm_pos[0] += rc.ch2 * ARM_RC_MOVE_RATIO_Y / RC_MAX_VALUE;
	arm_pos[1] += rc.ch4 * ARM_RC_MOVE_RATIO_Z / RC_MAX_VALUE;
		arm_pos_ctrl(arm_pos);
	}
						//servo_ctrl(&servo);

	if(last_rc_sw != rc.sw1) {
		last_rc_sw = rc.sw1;
		switch(rc.sw1) {
			case RC_UP:   //auto delive
			{	
			    servo.s_s3=ARM_S3_MIN;
					servo_ctrl(&servo);
					osDelay(500);
				
				  servo.s_s4 = ARM_S4_MAX;
      		servo_ctrl(&servo);
					osDelay(1500);

				
				  arm_pos[0]=150;
			    arm_pos[1]=100;
			    arm_pos_ctrl(arm_pos);
					osDelay(1500);

			    
     			servo.s_s3 = 0;
  				servo_ctrl(&servo);
					osDelay(500);
				
					servo.s_s3 = ARM_S3_MAX;
					servo_ctrl(&servo);
					osDelay(1500);
								
					arm_pos[0]=ARM_MIN_Y;
			    arm_pos[1]=ARM_MAX_Z;
			    arm_pos_ctrl(arm_pos);
					osDelay(500);

					servo.s_s4 = ARM_S4_MIN;
      		servo_ctrl(&servo);
					osDelay(500);

			    servo.s_s1=ARM_S1_STORE;
          servo.s_s2=ARM_S2_STORE;
					servo_ctrl(&servo);
					osDelay(500);

										type=0;

			    //osDelay(1500);
				}
			break;
			
			case RC_MI:
			{
							    arm_pos_ctrl(arm_pos);
					osDelay(500);

					servo.s_s3 = ARM_S3_MIN;
					servo_ctrl(&servo);
					osDelay(500);

//					servo.s_s4 = 1800;
//					servo_ctrl(&servo);
//			
					arm_pos[0] = 150;
						arm_pos_ctrl(arm_pos);
					arm_pos[1] = 60;
						arm_pos_ctrl(arm_pos);
				osDelay(200);
				
				
			    servo.s_s1=ARM_S1_STORE;
          servo.s_s2=ARM_S2_STORE;
					servo_ctrl(&servo);
					osDelay(500);
							    servo.s_s4 = ARM_S4_MIN;
      		servo_ctrl(&servo);
					osDelay(500);

										type=0;

				
			}
				break;
			
			case RC_DN:
			{
					servo.s_s4 = ARM_S4_MIN;
					servo_ctrl(&servo);
					osDelay(200);

					servo.s_s3 = ARM_S3_MAX;
					servo_ctrl(&servo);
					osDelay(500);
			
					arm_pos[0] = 130;
						arm_pos_ctrl(arm_pos);
					arm_pos[1] = 30;
						arm_pos_ctrl(arm_pos);
				osDelay(500);
					arm_pos[0] = ARM_MIN_Y;
						arm_pos_ctrl(arm_pos);
					arm_pos[1] = ARM_MIN_Z;
						arm_pos_ctrl(arm_pos);
				type=0;

			}
				break;
			
			case 4: //test
			{
         arm_pos_ctrl(arm_pos);
					osDelay(200);
			}
			break;
			
      case 5:
			{
			servo_ctrl(&servo);
			}
				break;
		}
	}
}

void arm_task(const void* argu)
{
	arm_init();
	while (1)
	{
		//arm_pos_ctrl(arm_pos);
		servo.s_camera = rc.wheel;
		servo_ctrl(&servo);

		osDelay(10);
	}
}
