/****************************************************************************
 *  Copyright (C) 2018 RoboMaster.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

/**
 *********************** (C) COPYRIGHT 2018 DJI **********************
 * @update
 * @history
 * Version     Date              Author           Modification
 * V1.3.0      1-30-2019   QiayuanLiao
 * @verbatim
 *********************** (C) COPYRIGHT 2018 DJI **********************
 */

#include "can_device.h"
#include "execute_task.h"
#include "chassis_task.h"
#include "arm_task.h"
#include "keyboard.h"
#include "math.h"
#include "pid.h"
#include "stdlib.h"
#include "string.h"
#include "sys.h"
#include "trajectory.h"
#include "uart_device.h"
#include "cmsis_os.h"

/* 底盘电机期望转速(rpm) */
int32_t chassis_moto_speed_ref[4];
/* 底盘电机电流 */
int16_t chassis_moto_current[4];

float chassis_delta_pos[4];
float angle_control_output;
int16_t LightCtrlSpeed=0;
int angle;
chassis_t chassis;

pid_t pid_angle_control;
//pid_t pid_bangbang_control;

/**
 * @brief     initialize chassis motor pid parameter
 * @usage     before chassis loop use this function
 */
void chassis_control_param_init(void) {
  //底盘PID参数设置
	for (int k = 0; k < 4; k++) {
		//pid_traj_init(&pid_chassis_traj[k], 16384, 32.0f, 0.1f, 1.0f, 0.2f);
		pid_init(&pid_wheel_spd[k], 16384, 20.0f, 7.5f, 0.02f, 2.0f);
  }
	pid_init(&pid_angle_control, MAX_CHASSIS_VW_SPEED, 30.0f, 15.0f, 1.0f, 2.0f);
	//pid_init(&pid_bangbang_control, MAX_CHASSIS_VX_SPEED, 0.0f, 100.0f, 0.0f, 0.0f);
}

void Traj_Move_Start(){
	//if(is_moving == 0){
		//is_moving = 1;
		trajectory_gener(&forward_traj, 600, MAX_CHASSIS_AY,MAX_CHASSIS_VY_SPEED);
		float t = 0.0f;
		uint32_t chassis_wake_time = osKernelSysTick();
		while(t<=forward_traj.time_acc){
			t += 0.01f;
			chassis.vy = trajectory_get_speed(&forward_traj,t);
			osDelayUntil(&chassis_wake_time, 10);
		}
	//}
}

void Traj_Move_Stop(){
	//if(is_moving == 1){
		trajectory_gener(&forward_traj, 600, MAX_CHASSIS_AY,MAX_CHASSIS_VY_SPEED);
		float t = forward_traj.time_total - forward_traj.time_acc;
		uint32_t chassis_wake_time = osKernelSysTick();
		while(t<=forward_traj.time_total){
			t += 0.01f;
			chassis.vy = trajectory_get_speed(&forward_traj,t);
			osDelayUntil(&chassis_wake_time, 10);
		}
		//is_moving = 0;
	//}

}

/*
void chassis_pid_angle_and_speed_get(void) {
  for (int k = 0; k < 4; k++) {
    //获得底盘电机角度
    pid_chassis_traj[k].get[0] = moto_chassis[k].total_angle;
    //获得底盘电机角速度
    pid_chassis_traj[k].get[1] = moto_chassis[k].speed_rpm*6;
  }
}

void chassis_traj_close_loop_calculate(void) {
  for (int i = 0; i < 4; i++) {
    pid_traj_calc(&pid_chassis_traj[i], pid_chassis_traj[i].get, pid_chassis_traj[i].set);
    chassis_moto_current[i] = pid_chassis_traj[i].out;
  }
}

void move_normal(float dx, float dy, float dw){
	static float rotate_ratio_f = ((WHEELBASE+WHEELTRACK)/2.0f - GIMBAL_OFFSET)/RADIAN_COEF;
  static float rotate_ratio_b = ((WHEELBASE+WHEELTRACK)/2.0f + GIMBAL_OFFSET)/RADIAN_COEF;
  static float wheel_mm_ratio = 360.0f/(PERIMETER*CHASSIS_DECELE_RATIO);

	chassis_delta_pos[0] = (+dx - dy + dw * rotate_ratio_f) * wheel_mm_ratio;
  chassis_delta_pos[1] = (+dx + dy + dw * rotate_ratio_f) * wheel_mm_ratio;
  chassis_delta_pos[2] = (-dx + dy + dw * rotate_ratio_b) * wheel_mm_ratio;
  chassis_delta_pos[3] = (-dx - dy + dw * rotate_ratio_b) * wheel_mm_ratio;
	for(int k =0; k < 4; k++){
		trajectory_gener(&chassis_traj[k],chassis_delta_pos[k],WHEEL_ACC,WHEEL_SPEED_NORMAL);
	}

	float t = 0.0f;
	uint32_t chassis_wake_time = osKernelSysTick();
	while(t<=chassis_traj[0].time_total || t<=chassis_traj[1].time_total || t<=chassis_traj[2].time_total || t<=chassis_traj[3].time_total){
		t += 0.01f;
		if(needStop_command){
			write_uart(USER_UART3, (uint8_t *)"OK" ,2);
			write_uart(USER_UART3, (uint8_t *)"OK" ,2);
			write_uart(USER_UART3, (uint8_t *)"OK" ,2);
			for (int k = 0; k < 4; k++){
				trajectory_stop(&chassis_traj[k], t);
			}
			needStop_command = 0;
		}
		for (int k = 0; k < 4; k++){
			trajectory_get(&chassis_traj[k],pid_chassis_traj[k].set,t);
		}
		osDelayUntil(&chassis_wake_time, 10);
	}
	for(int k = 0; k<4; k++){
		pid_traj_reflash(&pid_chassis_traj[k],moto_chassis[k].total_angle);
	}
	osDelay(200);
}
*/

/* 底盘控制信号获取 */
void chassis_angle_control_get(){
	float err = chassis.yaw - single_gyro.angle;
	angle_control_output = ANGLE_KP * err;
}

void chassis_bangbang_control_get() {
//	if(isRotating()) enable = 0;
//	if(!enable) {
//		bangbang_control_output = 0;
//		return;
	
//	}
//	if(!isRotating()) enable = 0;
}

void chassis_proto_information_get(void){
	chassis_angle_control_get();

  //底盘速度分解，计算底盘电机转速
  chassis_moto_speed_calc(chassis.vx + bangbang_control_output, chassis.vy, chassis.vw + angle_control_output, chassis_moto_speed_ref);
}

void chassis_remote_information_get(void)
{
  //遥控器以及鼠标对底盘的控制信息转化为标准单位，平移为(mm/s)旋转为(degree/s)
  chassis.vx = rc.ch1 * CHASSIS_RC_MOVE_RATIO_X / RC_MAX_VALUE * MAX_CHASSIS_VX_SPEED;
  chassis.vy = rc.ch2 * CHASSIS_RC_MOVE_RATIO_Y / RC_MAX_VALUE * MAX_CHASSIS_VY_SPEED;
  chassis.vw = rc.ch3 * CHASSIS_RC_MOVE_RATIO_R / RC_MAX_VALUE * MAX_CHASSIS_VW_SPEED;
//	
//	//Set Dest r,c to Avoid Going
	DestR = pose.r;
	DestC = pose.c;
	isGoing = isCanceling = 0;
//	
	  //底盘速度分解，计算底盘电机转速
  chassis_moto_speed_calc(chassis.vx , chassis.vy, chassis.vw, chassis_moto_speed_ref);
}

void chassis_arm_information_get(void) {
	chassis.vx = rc.ch1 * CHASSIS_RC_MOVE_RATIO_X / RC_MAX_VALUE * MAX_CHASSIS_VX_SPEED * 0.5f;
	chassis.vy = 0;
	chassis.vw = 0;
	
	DestR = pose.r;
	DestC = pose.c;
	isGoing = isCanceling = 0;

	//底盘速度分解，计算底盘电机转速
  chassis_moto_speed_calc(chassis.vx, chassis.vy, chassis.vw, chassis_moto_speed_ref);
}


/* 底盘运动的速度分解，以及电机转速的闭环控制 */
void chassis_speed_control(void)
{
  //闭环计算底盘轮子电机电流
  chassis_close_loop_calculate();

  //将计算好的电流值发送给电调
  send_chassis_moto_current(chassis_moto_current);
}

/* 底盘的运动分解处理 */
/**
  * @param 输入参数1: vx左右平移速度(mm/s)，右为正方向
  *        输入参数2: vy前后平移速度(mm/s)，前为正方向
  *        输入参数3: vw底盘旋转速度(degree/s)，逆时针为正方向
  *        输入参数4: speed[] 4个电机转速(rpm)
  * @note  下面是电机轮子编号，左上角为0号
  * @map   1 %++++++% 0
               ++++
               ++++
           2 %++++++% 3
  */
void chassis_moto_speed_calc(float vx, float vy, float vw, int32_t speed[])
{
  static float rotate_ratio_f = ((WHEELBASE+WHEELTRACK)/2.0f - GIMBAL_OFFSET)/RADIAN_COEF;
  static float rotate_ratio_b = ((WHEELBASE+WHEELTRACK)/2.0f + GIMBAL_OFFSET)/RADIAN_COEF;
  static float wheel_rpm_ratio = 60.0f/(PERIMETER*CHASSIS_DECELE_RATIO);

  int32_t wheel_rpm[4];
  float max = 0;

  VAL_LIMIT(vx, -MAX_CHASSIS_VX_SPEED, MAX_CHASSIS_VX_SPEED);  //mm/s
  VAL_LIMIT(vy, -MAX_CHASSIS_VY_SPEED, MAX_CHASSIS_VY_SPEED);  //mm/s
  VAL_LIMIT(vw, -MAX_CHASSIS_VW_SPEED, MAX_CHASSIS_VW_SPEED);  //deg/s
  
  wheel_rpm[0] = (+vx - vy + vw * rotate_ratio_f) * wheel_rpm_ratio;
  wheel_rpm[1] = (+vx + vy + vw * rotate_ratio_f) * wheel_rpm_ratio;
  wheel_rpm[2] = (-vx + vy + vw * rotate_ratio_b) * wheel_rpm_ratio;
  wheel_rpm[3] = (-vx - vy + vw * rotate_ratio_b) * wheel_rpm_ratio;

  //find max item
  for (uint8_t i = 0; i < 4; i++)
  {
    if (abs(wheel_rpm[i]) > max)
      max = abs(wheel_rpm[i]);
  }
  //equal proportion
  if (max > MAX_WHEEL_RPM)
  {
    float rate = MAX_WHEEL_RPM / max;
    for (uint8_t i = 0; i < 4; i++)
      wheel_rpm[i] *= rate;
  }
  memcpy(speed, wheel_rpm, 4*sizeof(int32_t));
}

void chassis_close_loop_calculate(void)
{
  for (int i = 0; i < 4; i++)
  {
    chassis_moto_current[i] = pid_calc(&pid_wheel_spd[i], moto_chassis[i].speed_rpm, chassis_moto_speed_ref[i]);
  }
}
