/****************************************************************************
 *  Copyright (C) 2018 RoboMaster.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

/**
  *********************** (C) COPYRIGHT 2018 DJI **********************
  * @update
  * @history
  * Version     Date              Author           Modification
  * V1.0.0      January-15-2018   ric.luo
  * @verbatim
  *********************** (C) COPYRIGHT 2018 DJI **********************
  */

#include "execute_task.h"
#include "chassis_task.h"
#include "detect_task.h"
#include "can_device.h"
#include "uart_device.h"
#include "pid.h"
#include "sys.h"

#include "stdlib.h"
#include "can.h"
#include "cmsis_os.h"
#include "string.h"
#include "arm_task.h"

#include "trajectory.h"

void chassis_task(const void *argu)
{
	int ctr = 0;

	chassis_control_param_init();
	uint32_t chassis_wake_time = osKernelSysTick();
	while (1)
	{
		get_chassis_mode();
		if(++ctr >= 10) {
			ctr = 0;
			pose_upload();
		}

		switch (chassis.mode)
		{
		case CHASSIS_PROTO:
		{
			chassis_proto_information_get();
			chassis_speed_control();
		}
		break;

		case CHASSIS_REMOTE:
		{
			chassis_remote_information_get();
			chassis_speed_control();
		}
		break;

		case CHASSIS_ARM:{
			chassis_arm_information_get();
			chassis_speed_control();
		}
		break;
		}

		osDelayUntil(&chassis_wake_time, CHASSIS_PERIOD);
	}
}

uint32_t Last_UpTick = 0;

void get_chassis_mode(void)
{
	chassis.last_mode = chassis.mode;
	switch (rc.sw2)
	{
	case RC_UP:
		chassis.mode = CHASSIS_PROTO;
		break;

	case RC_MI:
		chassis.mode = CHASSIS_REMOTE;
		break;

	case RC_DN:
		chassis.mode = CHASSIS_ARM;
		break;
	}
	
	//SINGLE GYRO OFFSETING
	if(rc.sw1 == RC_UP) Last_UpTick = osKernelSysTick();
	else if(rc.sw1 == RC_DN) {
		if( (osKernelSysTick() - Last_UpTick) <= 500 ) {
			Last_UpTick = 0;

			GyroAngleOffset = GyroAngleReal;
			My_Yaw = 0;
		}
	}
}