#include "execute_task.h"
#include "chassis_task.h"
#include "arm_task.h"

#include "can_device.h"
#include "uart_device.h"
#include "cmsis_os.h"
#include "calibrate.h"
#include "pid.h"
#include "sys.h"

#include <math.h>
#include <string.h>

////////////////DATA UPLOAD////////////////

float bangbang_control_output = 0;
//float last_bangbang_output = 0;
uint8_t bangbang_cross_line = 0;

uint8_t data_recv[PROCOTOL_LEN + 5];
uint8_t bangbang_recv[9];
uint8_t upload_buffer[UPLOAD_LEN + 5];

uint8_t DestR, DestC, DestDir = 0;
uint8_t Initiaized = 0;

////////////////RFID & POSE////////////////

uint32_t RFID_Modify_Cnt = 0;
struct RobotPose pose;

uint8_t LastCrossLine = 0;
uint32_t SF_CrossLineUsed = 0;
uint32_t SF_LastCorrectTime = 0;
uint32_t SF_LastCrosslineTime = 0;

////////////////Track Planning////////////////

int32_t yaw_direction[4] = {0, 180, 270, 90};
const int dir_dr[4] = {1, -1, 0, 0};
const int dir_dc[4] = {0, 0, -1, 1};

uint32_t Cancel_Modify_Cnt;
uint8_t isGoing, isCanceling;
uint8_t GoR, GoC, GoDir;

int32_t My_Yaw = 0;

/*
void Traj_Move_Start() {
	chassis_procotol.vy = 300;
}

void Traj_Move_Stop() {
	chassis_procotol.vy = 0;
}
*/

uint8_t isRotating(void) {
	return (fabsf(chassis.yaw - single_gyro.angle) > CHASSIS_YAW_DELTA) ? 1 : 0;
}

int8_t DirFromYaw(int32_t Yaw) {
	Yaw = (Yaw % 360 + 360) % 360;
	for(uint8_t i = 0; i < 4; i++) if(yaw_direction[i] == Yaw) {
		return i;
	}
	return 0;
}

int32_t DoCalcYaw(int32_t CurYaw, uint8_t DestDir) {
	if(DirFromYaw(CurYaw + 90) == DestDir) return CurYaw + 90;
	if(DirFromYaw(CurYaw - 90) == DestDir) return CurYaw - 90;
	if(DirFromYaw(CurYaw - 180) == DestDir) return CurYaw - 180;
	return CurYaw;
}

void auto_controller(void) {
	//Get Instruction Dir
	if((DestR != pose.r) && (DestC != pose.c)) {
		DestR = pose.r;
		DestC = pose.c;
		DestDir = pose.dir;
	}
	else {
		if(DestR > pose.r) DestDir = 0;
		else if(DestR < pose.r) DestDir = 1;
		else if(DestC < pose.c) DestDir = 2;
		else if(DestC > pose.c) DestDir = 3;
		else DestDir = pose.dir;
	}

	//Zero-ize speed
	chassis.vx = 0;
	chassis.vw = 0;

	//Initialize Position
	if(!Initiaized) {
		if(RFID_Modify_Cnt > 0) {
			Initiaized = 1;

			DestR = pose.r;
			DestC = pose.c;

			for(int i = 1; i <= 10; i++) pose_upload();
			osDelay(100);
		}
		return;
	}
	
	//sensor fusion
	/*if(bangbang_cross_line) {
		if(SF_LastCrosslineTime == 0) {
			SF_LastCrosslineTime = osKernelSysTick();
		}
	}
	else {
		SF_LastCrosslineTime = 0;
	}*/
	
	if(bangbang_cross_line != LastCrossLine) {
		/*uint8_t TimeValid = 
			( (SF_LastCrosslineTime != 0)
		&&  ( (osKernelSysTick() - SF_LastCrosslineTime) >= SF_CROSS_FILTER_DELAY)
			);*/

		if( bangbang_cross_line /*&& TimeValid*/ ) {
			if((osKernelSysTick() - SF_LastCorrectTime) >= SF_RFID_DELAY) {
				if(SF_CrossLineUsed < 2) {
					SF_CrossLineUsed++;

					pose.r += dir_dr[pose.dir];
					pose.c += dir_dc[pose.dir];

					SF_LastCorrectTime = osKernelSysTick();
					RFID_Modify_Cnt++;
				}
			}
		}
		LastCrossLine = bangbang_cross_line;
	}
	
	//if need to cancel, wait till next RFID
	if(isCanceling) {
		if(RFID_Modify_Cnt > Cancel_Modify_Cnt) {
			if(pose.dir == 0 || pose.dir == 1) osDelay(STOP_Y_DELAY);
			Traj_Move_Stop();

			//Canceling OK!
			isGoing = 0;
			isCanceling = 0;
		}
		return;
	}

	//if going
	if(isGoing) {
		//if dest changed, then cancel to next rfid
		if((DestR != GoR) || (DestC != GoC) || (DestDir != GoDir)) {
			Cancel_Modify_Cnt = RFID_Modify_Cnt;

			isCanceling = 1;
			return;
		}

		//if arrived dest, then stop
		if((pose.r == GoR) && (pose.c == GoC)) {
			if(pose.dir == 0 || pose.dir == 1) osDelay(STOP_Y_DELAY);
			Traj_Move_Stop();

			isGoing = 0;
			return;
		}
		return;
	}

	//if angle not setted wait for pid
	My_Yaw = DoCalcYaw(My_Yaw, DestDir);
	chassis.yaw = My_Yaw;

	if(isRotating()) return; //(if rotating, no action)

	//if different coordinate
	if( (DestR != pose.r) || (DestC != pose.c) || (DestDir != pose.dir) ) {
		//angle set then go
		GoR = DestR;
		GoC = DestC;
		GoDir = DestDir;

		isGoing = 1;

		Traj_Move_Start();
	}
}

////////////////////////////////////////////////
void bangbang_callback() {
	uint16_t b1 = *((uint16_t *) &bangbang_recv[0]);
	uint16_t b2 = *((uint16_t *) &bangbang_recv[2]);
	if(b1 != b2) return;

	uint16_t one = 1;
	
	int num_black = 0;
	for(uint16_t i = 0; i < 16; i++) num_black += (b1 >> i) & one;

	//OUTPUT CONTROL VALUE
	bangbang_cross_line = num_black >= 4;

	if(bangbang_cross_line) bangbang_control_output = 0;
	else if( (b1 >> 7) & one ) {
		bangbang_control_output = -BANGBANG_KP_LOW;
	}
	else if( (b1 >> 8) & one ){
		bangbang_control_output = BANGBANG_KP_LOW;
	}

	else if( (b1 >> 6) & one ){
		bangbang_control_output = -BANGBANG_KP;
	}
	else if( (b1 >> 9) & one ){
		bangbang_control_output = BANGBANG_KP;
	}
	
	else if( (b1 >> 5) & one ){
		bangbang_control_output = -BANGBANG_KP;
	}
	else if( (b1 >> 10) & one ){
		bangbang_control_output = BANGBANG_KP;
	}
	
	else if( (b1 >> 4) & one ){
		bangbang_control_output = -BANGBANG_KP;
	}
	else if( (b1 >> 11) & one ){
		bangbang_control_output = BANGBANG_KP;
	}
	
	else if( (b1 >> 3) & one ){
		bangbang_control_output = -BANGBANG_KP;
	}
	else if( (b1 >> 12) & one ){
		bangbang_control_output = BANGBANG_KP;
	}
	
	else if( (b1 >> 2) & one ){
		bangbang_control_output = -BANGBANG_KP;
	}
	else if( (b1 >> 13) & one ){
		bangbang_control_output = BANGBANG_KP;
	}
	
	else if( (b1 >> 1) & one ){
		bangbang_control_output = -BANGBANG_KP;
	}
	else if( (b1 >> 14) & one ){
		bangbang_control_output = BANGBANG_KP;
	}

	else if( (b1 >> 0) & one ){
		bangbang_control_output = -BANGBANG_KP;
	}
	else if( (b1 >> 15) & one ){
		bangbang_control_output = BANGBANG_KP;
	}

	else {
		bangbang_control_output = 0;
	}
}

void usart_initialize() {
  uart_init(USER_UART3, 9600, WORD_LEN_8B, STOP_BITS_1, PARITY_NONE);

  uart_recv_callback_register(USER_UART3, usart_callback);
  uart_receive_start(USER_UART3, data_recv, PROCOTOL_LEN);
}

void ir_initialize() {
  uart_init(USER_UART4, 115200, WORD_LEN_8B, STOP_BITS_1, PARITY_NONE);

  uart_recv_callback_register(USER_UART4, bangbang_callback);
  uart_receive_start(USER_UART4, bangbang_recv, 4);
}

void RFID_ReadHandler(uint8_t* p_buf, uint16_t len)
{
    if(p_buf != NULL )
    {
				int rf_r = 12 - p_buf[0 + EXT_HEADER_HEAD_CMDID_LEN] + 1;
				int rf_c = p_buf[1 + EXT_HEADER_HEAD_CMDID_LEN];

				if((rf_r != pose.r) || (rf_c != pose.c))
					RFID_Modify_Cnt++;

				pose.r = rf_r;
				pose.c = rf_c;
				
				SF_CrossLineUsed = 0;
				SF_LastCorrectTime = osKernelSysTick();
    }
}

void rfid_decode() {
	ExtProtocolUnpack(&EXT_protocol_Obj);
}

void execute_task(const void* argu)
{
	usart_initialize();
	ir_initialize();

	while(1) {
		if(chassis.mode == CHASSIS_PROTO){
			if(chassis.mode != CHASSIS_PROTO){
				DestR = pose.r;
				DestC = pose.c;
				for(int i = 1; i <= 10; i++) pose_upload();
			}
			auto_controller();
		}
		rfid_decode();
		osDelay(10);
	}
}

uint8_t checksum(uint8_t *data, uint8_t len) {
	uint8_t sum = 0;
	for(int i = 0; i < len - 1; i++) sum += data[i];

	return (sum == data[len - 1]);
}

void usart_callback(void) {
	if(!checksum(data_recv, PROCOTOL_LEN)) return;

	DestR =    data_recv[0];
	DestC =    data_recv[1];
	//DestDir =  data_recv[2];
}

void pose_upload(void) {
	pose.dir = DestDir;

	int checksum = (int)pose.r + (int)pose.c + (int)pose.dir;
	sprintf((char *)upload_buffer, "[[%d %d %d %d]]", (int)pose.r, (int)pose.c, (int)pose.dir, checksum);

	write_uart( USER_UART3, upload_buffer, strlen((const char *)upload_buffer) );
}
