/****************************************************************************
 *  Copyright (C) 2018 RoboMaster.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

/**
  *********************** (C) COPYRIGHT 2018 DJI **********************
  * @update
  * @history
  * Version     Date              Author           Modification
  * V1.0.0      January-15-2018   ric.luo
  * @verbatim
  *********************** (C) COPYRIGHT 2018 DJI **********************
  */ 
 



#ifndef __SYS_H__
#define __SYS_H__

#include "rm_hal_lib.h"

/* 4轮车，如果使用3轮车注释掉此宏定义 */
#define FOUR_WHEEL


/*************************底盘位置设置*******************************/


#define RADIAN_COEF 57.2974f
#define WHEEL_ACC	500.0f*36.0f
#define WHEEL_SPEED_NORMAL 600.0f*36.0f

/*************************底盘速度设置*******************************/

/* 遥控器模式下的底盘最大速度限制 */
/* 底盘平移速度 */
#define CHASSIS_RC_MOVE_RATIO_X 1.0f
/* 底盘前进速度 */
#define CHASSIS_RC_MOVE_RATIO_Y 1.0f
/* 底盘旋转速度，只在底盘开环模式下使用 */
#define CHASSIS_RC_MOVE_RATIO_R 1.0f

/* 鼠标键盘模式下的底盘最大速度限制 */
/* 底盘平移速度 */
#define CHASSIS_PC_MOVE_RATIO_X 1.0f
/* 底盘前进速度 */
#define CHASSIS_PC_MOVE_RATIO_Y 1.0f
/* 底盘旋转速度，只在底盘开环模式下使用 */
#define CHASSIS_PC_MOVE_RATIO_R 5.0f

/*************************云台速度设置*******************************/

/* 遥控器模式下的云台速度限制 */
/* 云台pitch轴速度 */
#define GIMBAL_RC_MOVE_RATIO_PIT 1.0f
/* 云台yaw轴速度 */
#define GIMBAL_RC_MOVE_RATIO_YAW 1.0f
/* 鼠标键盘模式下的云台速度限制 */
/* 云台pitch轴速度 */
#define GIMBAL_PC_MOVE_RATIO_PIT 1.2f
/* 云台yaw轴速度 */
#define GIMBAL_PC_MOVE_RATIO_YAW 1.0f


/*************************发射速度设置*******************************/
#define SHOT_FRIC_WHEEL_SPEED    1300 //最大为2500


/*************************发射频率设置*******************************/
#define TRIGGER_MOTOR_SPEED      -5000 //



/*************************机械臂设置*******************************/

#define  ARM_S1_IO     PWM_IO1
#define  ARM_S2_IO     PWM_IO2
#define  ARM_S3_IO     PWM_IO3
#define  ARM_S4_IO     PWM_IO4
#define  ARM_S5_IO     PWM_IO5


#define  ARM_S1_INIT      500
#define  ARM_S2_INIT      800
#define  ARM_S3_INIT      1300
#define ARM_S4_INIT       500


#define ARM_L1 120
#define ARM_L2 120

#define SERVO_MAX_ANGLE 280
#define  ARM_S3_MAX_ANGLE      1800
#define  ARM_S3_MIN_ANGLE      500
#define  ARM_CLAW_OPEN_ANGLE      1000
#define  ARM_CLAW_CLOSE_ANGLE      500

/* 遥控器的最大行程 */
#define RC_MAX_VALUE      660.0f

#define RC_RATIO          0.002f
#define KB_RATIO          0.02f

/* 单发拨弹的编码器行程 */
#define DEGREE_60_TO_ENCODER 	 -49146


/********** 4轮底盘信息 **********/
/* 底盘轮距(mm) */
#define WHEELTRACK     250.0f
/* 底盘轴距(mm) */
#define WHEELBASE      250.0f
/* 云台偏移(mm) */
#define GIMBAL_OFFSET  0.0f

#define PERIMETER  239.582f
#define CHASSIS_PERIOD 10

/******** 底盘最大速度设置 *******/
/* 底盘移动最大速度，单位是毫米每秒 */
#define MAX_CHASSIS_VX_SPEED 500.0f
#define MAX_CHASSIS_VY_SPEED 500.0f
/* 底盘旋转最大速度，单位是度每秒 */
#define MAX_CHASSIS_VW_SPEED 180.0f

#define SF_RFID_DELAY 500.0f
#define MAX_CHASSIS_AY 1000.0f

#define SF_CROSS_FILTER_DELAY 10
#define STOP_Y_DELAY   10

#define ANGLE_KP 10.0f
#define BANGBANG_KP 100.0f
#define BANGBANG_KP_LOW 10.0f

/******** 底盘电机使用2006 *******/
/* 2006底盘电机减速比 */
#define CHASSIS_DECELE_RATIO (1.0f/36.0f)
/* 单个电机速度极限，单位是分钟每转 */
#define MAX_WHEEL_RPM        5000   //8347rpm = 3500mm/s

/* 串口数据相关 */
#define MAX_DMA_COUNT        200
#define DBUS_FRAME_SIZE      18

/* 常用的一些物理系数 */
#define DGR_RAD 57.3f

#define PI 3.141592f
/* 极值限制函数宏定义 */
#define VAL_LIMIT(val, min, max)\
if((val) <= (min))\
{\
  (val) = (min);\
}\
else if((val) >= (max))\
{\
  (val) = (max);\
}\

#endif
